@@ -38,7 +38,7 @@ class Log():
 
     # makes dir
     def makeDirIfNotExist(self, path):
-        if (path is not ""):
+        if (path != ""):
             try:
                 os.makedirs(path)
             except OSError as e:
