@@ -1,7 +1,12 @@
 #!/usr/bin/env python3
 import gi
 gi.require_version("Gtk", "3.0")
-gi.require_version('Budgie', '1.0')
+gi.require_version('Libxfce4windowing', '0.0')
+from gi.repository import Libxfce4windowing
+if Libxfce4windowing.windowing_get() == Libxfce4windowing.Windowing.WAYLAND:
+    gi.require_version('Budgie', '2.0')
+else:
+    gi.require_version('Budgie', '1.0')
 from gi.repository import Gdk, Gtk, GObject, GdkPixbuf, Budgie, Gio, GLib
 import os
 import time
