@@ -3,7 +3,7 @@
 """
 Count Down
 Author: Jacob Vlijm
-Copyright © 2017-2020 Ubuntu Budgie Developers
+Copyright © 2017 Ubuntu Budgie Developers
 Website: https://ubuntubudgie.org
 This program is free software: you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -15,13 +15,12 @@ should have received a copy of the GNU General Public License along with this
 program.  If not, see <http://www.gnu.org/licenses/>.
 """
 
+
 import os
-
 import gi
-
 gi.require_version('Budgie', '1.0')
 gi.require_version('Gtk', '3.0')
-from gi.repository import Budgie, GObject, GdkPixbuf, Gtk, Gio
+from gi.repository import Budgie, GObject, GdkPixbuf, Gtk, Gio, GLib
 from threading import Thread
 import time
 import subprocess
@@ -74,21 +73,72 @@ class CountDown(GObject.GObject, Budgie.Plugin):
         return CountDownApplet(uuid)
 
 
+class CountDownSettings(Gtk.Grid):
+
+    def __init__(self, setting):
+
+        super().__init__()
+        self.show_seconds = True
+        self.setting = setting
+        # grid & layout
+        countdown_spacegrid = Gtk.Grid()
+        self.add(countdown_spacegrid)
+        for cell in [[0, 0], [100, 0], [0, 100], [100, 100]]:
+            countdown_spacegrid.attach(
+                Gtk.Label(label="\t"), cell[0], cell[1], 1, 1
+            )
+        self.space_settings = Gio.Settings(
+            schema="org.ubuntubudgie.plugins.budgie-countdown"
+        )
+        currvalue = self.space_settings.get_int("spacersize")
+        space = Gtk.SpinButton()
+        space.set_range(0, 50)
+        space.set_increments(1, 1)
+        space.set_value(currvalue)
+        space.connect("value-changed", self.update_value)
+        label = Gtk.Label("Built-in spacer" + "\n")
+        label.set_xalign(0)
+        countdown_spacegrid.attach(label, 1, 1, 2, 1)
+        countdown_spacegrid.attach(space, 1, 2, 1, 1)
+        self.show_all()
+
+    def update_value(self, spin):
+        newval = spin.get_value()
+        self.space_settings.set_int("spacersize", newval)
+
+
 class CountDownApplet(Budgie.Applet):
     """ Budgie.Applet is in fact a Gtk.Bin """
 
     def __init__(self, uuid):
         Budgie.Applet.__init__(self)
+        self.uuid = uuid
         self.red_time = 60
         self.yellow_time = 300
-        # --- panelgrid ---
-        self.panelgrid = Gtk.Grid()
-        # space section. make it depend on panel size?
-        spacer_img = Gtk.Image.new_from_file(
-            os.path.join(path, "cr_spacer.png")
+        # setup watching applet presence
+        self.currpanelsubject_settings = None
+        GLib.timeout_add_seconds(1, self.watchout)
+        self.countdown_onpanel = True
+        # spacesettings
+        self.space_settings = Gio.Settings(
+            schema="org.ubuntubudgie.plugins.budgie-countdown"
         )
-        self.panelgrid.attach(spacer_img, 0, 0, 2, 1)
-        self.panelgrid.set_row_spacing(10)  # <-- make depend on panel height
+        self.claimed_panelspace = self.space_settings.get_int("spacersize")
+        self.space_settings.connect("changed", self.get_currclaimedspace)
+        # --- panelgrid/containergrid ---
+        self.panelgrid = Gtk.Grid()
+        self.panelgrid.set_row_spacing(5)
+        self.panelgrid.set_column_spacing(5)
+        self.containergrid = Gtk.Grid()
+        self.containergrid.attach(self.panelgrid, 1, 1, 1, 1)
+        # --- let's initiate some stuff ---
+        self.panelspacing = 0
+        self.position_index = 0
+        # label/spacer position
+        self.labelpos = [0, 0, 1, 1]
+        self.spacerpos = "top"
+        self.grid_helpers = []  # spacer images
+        self.labelplacement = 0  # distance
         # icons
         grey = os.path.join(path, "cr_grey.png")
         green = os.path.join(path, "cr_green.png")
@@ -101,10 +151,10 @@ class CountDownApplet(Budgie.Applet):
         ]
         # initial icon
         self.seticon = Gtk.Image.new_from_pixbuf(self.iconset[0])
-        self.panelgrid.attach(self.seticon, 0, 1, 1, 1)
+        self.panelgrid.attach(self.seticon, 1, 1, 1, 1)
         # menu label
-        self.timer = Gtk.Label("  00:00:00")
-        self.panelgrid.attach(self.timer, 1, 1, 1, 1)
+        self.timer = Gtk.Label(label="00:00:00")
+        self.panelgrid.attach(self.timer, 2, 1, 1, 1)
         # --- menugrid ---
         self.menugrid = Gtk.Grid()
         self.menugrid.set_column_spacing(15)
@@ -132,18 +182,14 @@ class CountDownApplet(Budgie.Applet):
         self.secsbutton = Gtk.SpinButton()
         self.secsbutton.set_adjustment(adjustment)
         self.menugrid.attach(self.secsbutton, 2, 4, 1, 1)
-
         for sp in [self.hoursbutton, self.minsbutton, self.secsbutton]:
             sp.set_numeric(True)
             sp.set_update_policy(True)
-
         # prevent pause
         self.sleep = Gtk.CheckButton("Prevent pausing countdown")
         self.menugrid.attach(self.sleep, 1, 6, 2, 1)
-
         sep = Gtk.Separator()
         self.menugrid.attach(sep, 4, 1, 1, 8)
-
         self.bbox = Gtk.Box()
         self.menugrid.attach(self.bbox, 0, 10, 9, 2)
         self.menugrid.attach(Gtk.Label(""), 1, 9, 1, 1)
@@ -160,7 +206,6 @@ class CountDownApplet(Budgie.Applet):
         self.menugrid.attach(self.nf_icon, 5, 3, 1, 1)
         self.nf_message = Gtk.CheckButton("Display window")
         self.menugrid.attach(self.nf_message, 5, 4, 1, 1)
-
         self.runcomm = Gtk.CheckButton("Run command:")
         self.menugrid.attach(self.runcomm, 5, 5, 1, 1)
         self.command_entry = Gtk.Entry()
@@ -207,17 +252,15 @@ class CountDownApplet(Budgie.Applet):
         self.cancel = True
         # panel
         self.box = Gtk.EventBox()
-        self.box.add(self.panelgrid)
+        self.box.add(self.containergrid)
         self.add(self.box)
         # menu
         self.popover = Budgie.Popover.new(self.box)
         self.popover.add(self.menugrid)
         self.popover.get_child().show_all()
-
         self.box.show_all()
         self.show_all()
         self.box.connect("button-press-event", self.on_press)
-
         GObject.threads_init()
         # thread
         self.update = Thread(target=self.run_countdown)
@@ -226,6 +269,111 @@ class CountDownApplet(Budgie.Applet):
         self.update.start()
         self.seticon.set_from_pixbuf(self.iconset[1])
 
+    def watchout(self):
+        path = "com.solus-project.budgie-panel"
+        panelpath_prestring = "/com/solus-project/budgie-panel/panels/"
+        panel_settings = Gio.Settings.new(path)
+        allpanels_list = panel_settings.get_strv("panels")
+        for p in allpanels_list:
+            panelpath = panelpath_prestring + "{" + p + "}/"
+            self.currpanelsubject_settings = Gio.Settings.new_with_path(
+                path + ".panel", panelpath
+            )
+            applets = self.currpanelsubject_settings.get_strv("applets")
+            if self.uuid in applets:
+                self.currpanelsubject_settings.connect(
+                    "changed", self.check_ifonpanel
+                )
+        return False
+
+    def check_ifonpanel(self, *args):
+        applets = self.currpanelsubject_settings.get_strv("applets")
+        self.countdown_onpanel = self.uuid in applets
+
+    def get_currclaimedspace(self, *args):
+        self.claimed_panelspace = self.space_settings.get_int("spacersize")
+        if self.spacerpos == "top":
+            self.containergrid.set_column_spacing(self.claimed_panelspace)
+        else:
+            self.containergrid.set_row_spacing(self.claimed_panelspace)
+
+    def do_get_settings_ui(self):
+        """Return the applet settings with given uuid"""
+        return CountDownSettings(self.get_applet_settings(self.uuid))
+
+    def do_supports_settings(self):
+        """Return True if support setting through Budgie Setting,
+        False otherwise.
+        """
+        return True
+
+    def do_panel_size_changed(self, panelsize, icsize, small_icsize):
+        diff = icsize - small_icsize
+        # 22 is the used icon size, 3 is an apparent constant
+        self.panelspacing = round(
+            ((panelsize - icsize) / 2) + ((icsize - 22) / 2) - 3
+        )
+        self.edit_grid()
+
+    def do_panel_position_changed(self, panelposition):
+        if panelposition != Budgie.PanelPosition.NONE:
+            panelpositions = [
+                Budgie.PanelPosition.RIGHT,
+                Budgie.PanelPosition.LEFT,
+                Budgie.PanelPosition.BOTTOM,
+                Budgie.PanelPosition.TOP,
+            ]
+            angles = [270, 90, 0, 0]
+            self.position_index = panelpositions.index(panelposition)
+            if self.position_index > 1:
+                self.labelpos = [2, 1, 1, 1]
+                self.spacerpos = "top"
+                self.timer.set_angle(0)
+            else:
+                self.labelpos = [1, 2, 1, 1]
+                self.spacerpos = "left"
+            self.timer.set_angle(angles[self.position_index])
+            self.edit_grid()
+
+    def edit_grid(self):
+        for img in self.grid_helpers:
+            self.containergrid.remove(img)
+        spacerpath = "/usr/share/pixmaps/cr_panelspacer.png"
+        self.grid_helpers = []
+        spacerimg = Gtk.Image.new_from_file(spacerpath)
+        built_in_spacer1 = Gtk.Image.new_from_file(spacerpath)
+        built_in_spacer2 = Gtk.Image.new_from_file(spacerpath)
+        for img in [
+            spacerimg, built_in_spacer1, built_in_spacer2,
+        ]:
+            self.grid_helpers.append(img)
+        if self.spacerpos == "left":
+            self.containergrid.set_margin_left(3)
+            self.containergrid.attach(spacerimg, 0, 1, 1, 1)
+            self.containergrid.attach(built_in_spacer1, 1, 0, 1, 1)
+            self.containergrid.attach(built_in_spacer2, 1, 2, 1, 1)
+            self.containergrid.set_column_spacing(self.panelspacing)
+            self.containergrid.set_row_spacing(self.claimed_panelspace)
+        else:
+            self.containergrid.set_margin_top(3)
+            self.containergrid.attach(spacerimg, 1, 0, 1, 1)
+            self.containergrid.attach(built_in_spacer1, 0, 1, 1, 1)
+            self.containergrid.attach(built_in_spacer2, 2, 1, 1, 1)
+            self.containergrid.set_row_spacing(self.panelspacing)
+            self.containergrid.set_column_spacing(self.claimed_panelspace)
+        # helper left or right, set size, add to self.grid_helpers
+        self.panelgrid.remove(self.timer)
+        if self.countdown != 0:
+            self.panelgrid.attach(
+                self.timer,
+                self.labelpos[0],
+                self.labelpos[1],
+                self.labelpos[2],
+                self.labelpos[3],
+            )
+        self.panelgrid.show_all()
+        self.containergrid.show_all()
+
     def get_setting(self, checkbox, readval=False):
         file = os.path.join(settingspath, checkbox[1])
         exists = os.path.exists(file)
@@ -281,7 +429,7 @@ class CountDownApplet(Budgie.Applet):
         hrs = self.two_dg(int(seconds / 3600))
         mins = self.two_dg(int((seconds % 3600) / 60))
         secs = self.two_dg(seconds % 60)
-        return "  " + ":".join([hrs, mins, secs])
+        return ":".join([hrs, mins, secs])
 
     def set_label(self, newlabel):
         GObject.idle_add(
@@ -388,7 +536,9 @@ class CountDownApplet(Budgie.Applet):
             self.context_start.remove_class(Gtk.STYLE_CLASS_SUGGESTED_ACTION)
             self.context_start.add_class(Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION)
             GObject.idle_add(
-                self.panelgrid.attach, self.timer, 1, 1, 1, 1,
+                self.panelgrid.attach, self.timer,
+                self.labelpos[0], self.labelpos[1],
+                self.labelpos[2], self.labelpos[3],
                 priority=GObject.PRIORITY_DEFAULT,
             )
             GObject.idle_add(
@@ -404,6 +554,7 @@ class CountDownApplet(Budgie.Applet):
 
     def handle_apply(self, button):
         set_t = self.get_settime()
+        self.show_seconds = True
         if self.countdown != 0:
             # cancelling countdown
             self.cancel = True
@@ -413,7 +564,7 @@ class CountDownApplet(Budgie.Applet):
         elif set_t != 0:
             # starting countdown, if set time > 0
             self.set_state(False)
-            self.countdown = set_t
+            self.countdown = set_t + 1
             self.init_t = time.time()
             # time shift to the real time
             self.diff = 0
@@ -435,8 +586,7 @@ class CountDownApplet(Budgie.Applet):
         cycle = 9
         self.currstate1 = None
         self.diff = 0
-
-        while True:
+        while self.countdown_onpanel:
             time.sleep(1 - self.diff)
             # see where we are in the time stages
             self.currstate2 = self.lookup_stage(self.countdown)
@@ -447,7 +597,8 @@ class CountDownApplet(Budgie.Applet):
             if self.countdown != 0:
                 self.countdown = self.countdown - 1
                 timelabel = self.calc_timedisplay(self.countdown)
-                self.set_label(timelabel)
+                if timelabel is not None:
+                    self.set_label(timelabel)
                 cycle = cycle + 1
                 if cycle == 10:
                     # once per 10 seconds, synchronize with real clock
