@@ -12,7 +12,7 @@ import subprocess
 """
 DropBy
 Author: Jacob Vlijm
-Copyright © 2017-2022 Ubuntu Budgie Developers
+Copyright © 2017 Ubuntu Budgie Developers
 Website=https://ubuntubudgie.org
 This program is free software: you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -108,8 +108,9 @@ class BudgieDropByApplet(Budgie.Applet):
         self.uuid = uuid
         self.connect("destroy", Gtk.main_quit)
         app_path = os.path.dirname(os.path.abspath(__file__))
-        user = os.environ["USER"]
-        self.copytrigger = "/tmp/" + user + "_dropby_icon_copy"
+        self.tmp_path = os.getenv("XDG_RUNTIME_DIR") \
+            if "XDG_RUNTIME_DIR" in os.environ else os.getenv("HOME")
+        self.copytrigger = os.path.join(self.tmp_path, ".dropby_icon_copy")
         self.copying = False
         self.winpath = os.path.join(app_path, "dropover")
         self.box = Gtk.EventBox()
@@ -133,7 +134,7 @@ class BudgieDropByApplet(Budgie.Applet):
         self.box.show_all()
         self.show_all()
         self.setup_watching()
-        self.start_dropover()
+        GLib.idle_add(self.start_dropover)
         self.refresh_from_idle()
 
     def set_icon_state(self):
@@ -188,13 +189,13 @@ class BudgieDropByApplet(Budgie.Applet):
 
     def create_windowtrigger(self, *args):
         if not self.check_winexists():
-            user = os.environ["USER"]
-            open("/tmp/" + user + "_call_dropby", "wt").write("")
+            open(os.path.join(self.tmp_path, ".call_dropby"), "wt").write("")
 
     def start_dropover(self):
+        user = os.environ["USER"]
         try:
             pid = subprocess.check_output(
-                ["/usr/bin/pgrep", "-f", self.winpath]
+                ["/usr/bin/pgrep", "-u", user, "-f", self.winpath]
             )
         except subprocess.CalledProcessError:
             subprocess.Popen([self.winpath, self.uuid])
